package cs202.account;

public class Account {

    private static int nextAccn = 1000;

    private int        accn;           // account number
    private String     owner;          // owner's name
    private double     balance;        // balance (>=0)

    /** contructor. create a zero-balance account */
    public Account( String owner )
    {
        // class variable owner is "shadowed" by the parameter owner, but
        // variables accn and balance are not.
        this.owner = owner;
        accn = nextAccn++;
        balance = 0.0;
    }

    /** constructor. creates an account with specified balance */
    public Account( String owner, double balance )
    {
        // calling another contructor using "this"
        this( owner );
        this.balance = balance > 0 ? balance : 0;
    }

    public double balance()
    {
        return balance;
    }

    // return the actual withdrawn amount
    public double withdraw( double amount )
    {
        double actualAmount = balance > amount ? amount : balance;
        balance -= actualAmount;
        return actualAmount;
    }

    // return the actual deposit amount
    public double deposit( double amount )
    {
        double actualAmount = amount > 0 ? amount : 0;
        balance += actualAmount;
        return actualAmount;
    }

    // return the actual transferred amount
    public double transfer( double amount, Account other )
    {
        // NOTE that if withdraw fails, it's no big deal, because
        // then withdraw resturns 0 and no harm is done to deposit
        // 0. However, we should be very concerned if withdraw
        // succeeds but deposit fails, in which case we have to
        // reimburse the withdrawn account.
        return other.deposit( withdraw( amount ) );
    }

    // overrides the toString method in Object
    public String toString()
    {
        return "Account #" + accn + "\n\tOwner:\t" + owner + "\n\tBalance:\t"
                + balance;
    }

    // tests
    public static void main( String args[] )
    {
        Account a1 = new Account( "Chengyu", 100 );
        Account a2 = new Account( "Sun" );

        System.out.println( "A1's balance is " + a1.balance() );
        System.out.println( "A2's balance is " + a2.balance() );

        a2.deposit( 20 );
        a1.transfer( 200, a2 );

        System.out.println( "A1's balance is " + a1.balance() );
        System.out.println( "A2's balance is " + a2.balance() );
    }

} // end of class Account
